@@ -3,6 +3,7 @@
 # ------------------------------------------------------------------------
 
 from django import forms
+from django.core.cache import cache as django_cache
 from django.conf import settings as django_settings
 from django.contrib import admin
 from django.core.urlresolvers import reverse
@@ -100,7 +101,6 @@ class PageManager(models.Manager):
         # Both cases can handle a short time of "being wrong", so this should
         # be OK.
 
-        from django.core.cache import cache as django_cache
         if settings.FEINCMS_USE_CACHE:
             ck = 'PAGE-FOR-URL-' + path
             page = django_cache.get(ck)
@@ -190,19 +190,20 @@ class Page(Base):
     title = models.CharField(_('title'), max_length=200,
         help_text=_('This is used for the generated navigation too.'))
     slug = models.SlugField(_('slug'), max_length=150)
-    parent = models.ForeignKey('self', blank=True, null=True, related_name='children')
+    parent = models.ForeignKey('self', verbose_name=_('Parent'), blank=True, null=True, related_name='children')
     parent.parent_filter = True # Custom list_filter - see admin/filterspecs.py
     in_navigation = models.BooleanField(_('in navigation'), default=True)
-    override_url = models.CharField(_('override URL'), max_length=400, blank=True,
+    override_url = models.CharField(_('override URL'), max_length=300, blank=True,
         help_text=_('Override the target URL. Be sure to include slashes at the beginning and at the end if it is a local URL. This affects both the navigation and subpages\' URLs.'))
-    redirect_to = models.CharField(_('redirect to'), max_length=400, blank=True,
+    redirect_to = models.CharField(_('redirect to'), max_length=300, blank=True,
         help_text=_('Target URL for automatic redirects.'))
-    _cached_url = models.CharField(_('Cached URL'), max_length=400, blank=True,
+    _cached_url = models.CharField(_('Cached URL'), max_length=300, blank=True,
         editable=False, default='', db_index=True)
 
     request_processors = []
     response_processors = []
-    cache_key_components = [ lambda p: p._django_content_type.id,
+    cache_key_components = [ lambda p: django_settings.SITE_ID,
+                             lambda p: p._django_content_type.id,
                              lambda p: p.id ]
 
     class Meta:
@@ -265,6 +266,11 @@ class Page(Base):
         cached_page_urls[self.id] = self._cached_url
         super(Page, self).save(*args, **kwargs)
 
+        # Okay, we changed the URL -- remove the old stale entry from the cache
+        if settings.FEINCMS_USE_CACHE:
+            ck = 'PAGE-FOR-URL-' + self._original_cached_url.strip('/')
+            django_cache.delete(ck)
+
         # If our cached URL changed we need to update all descendants to
         # reflect the changes. Since this is a very expensive operation
         # on large sites we'll check whether our _cached_url actually changed
@@ -328,6 +334,7 @@ class Page(Base):
         """
         request._feincms_page = self
         request._feincms_extra_context = {}
+        request.extra_path = ""
 
         for fn in self.request_processors:
             r = fn(self, request)
@@ -585,6 +592,7 @@ class PageAdminForm(forms.ModelForm):
 
         return cleaned_data
 
+
 # ------------------------------------------------------------------------
 if settings.FEINCMS_PAGE_USE_SPLIT_PANE_EDITOR:
     list_modeladmin = editor.SplitPaneEditor
@@ -594,17 +602,6 @@ else:
 # MARK: -
 # ------------------------------------------------------------------------
 
-# We'll create a replacement for the normal "delete_selected" action which is
-# slower, calling .delete() on each object rather than the queryset .delete()
-# but avoids problems with hierarchy leading to all pages being deleted.
-#
-# See the warning here:
-# http://docs.djangoproject.com/en/dev/ref/contrib/admin/actions/ for details
-def delete_page_action(modeladmin, request, queryset):
-    for i in queryset:
-        i.delete()
-delete_page_action.short_description = "Delete selected pages"
-
 class PageAdmin(editor.ItemEditor, list_modeladmin):
     class Media:
         css = {}
@@ -633,8 +630,6 @@ class PageAdmin(editor.ItemEditor, list_modeladmin):
     show_on_top = ['title', 'active', 'parent']
     radio_fields = {'template_key': admin.HORIZONTAL}
 
-    actions = [ delete_page_action ]
-
     def __init__(self, *args, **kwargs):
         if len(Page._feincms_templates) > 4:
             del(self.radio_fields['template_key'])
@@ -643,11 +638,6 @@ class PageAdmin(editor.ItemEditor, list_modeladmin):
 
     in_navigation_toggle = editor.ajax_editable_boolean('in_navigation', _('in navigation'))
 
-    def get_actions(self, request):
-        actions = super(PageAdmin, self).get_actions(request)
-        del actions['delete_selected']
-        return actions
-
     def _actions_column(self, page):
         actions = super(PageAdmin, self)._actions_column(page)
         actions.insert(0, u'<a href="add/?parent=%s" title="%s"><img src="%simg/admin/icon_addlink.gif" alt="%s"></a>' % (
@@ -697,6 +687,29 @@ class PageAdmin(editor.ItemEditor, list_modeladmin):
             self.message_user(request, ugettext("You have replaced %s. You may continue editing the now-active page below.") % page)
             return HttpResponseRedirect('.')
 
+        # Hack around a Django bug: raw_id_fields aren't validated correctly for
+        # ForeignKeys in 1.1: http://code.djangoproject.com/ticket/8746 details
+        # the problem - it was fixed for MultipleChoiceFields but not ModelChoiceField
+        # See http://code.djangoproject.com/ticket/9209
+
+        if hasattr(self, "raw_id_fields"):
+            for k in self.raw_id_fields:
+                if not k in request.POST:
+                    continue
+                if not isinstance(getattr(Page, k).field, models.ForeignKey):
+                    continue
+
+                v = request.POST[k]
+
+                if not v:
+                    del request.POST[k]
+                    continue
+
+                try:
+                    request.POST[k] = int(v)
+                except ValueError:
+                    request.POST[k] = None
+
         return super(PageAdmin, self).change_view(request, object_id, extra_context)
 
     def render_item_editor(self, request, object, context):
@@ -716,7 +729,6 @@ class PageAdmin(editor.ItemEditor, list_modeladmin):
         if not hasattr(self, "_visible_pages"):
             self._visible_pages = list() # Sanity check in case this is not already defined
 
-        # TODO: Couldn't this simply check page.get_ancestors(visible=False).count()?
         if page.parent_id and not page.parent_id in self._visible_pages:
             # parent page's invisibility is inherited
             if page.id in self._visible_pages:
