@@ -1,5 +1,12 @@
 from random import random
-import os, Image
+import os
+try:
+    from PIL import Image
+except ImportError:
+    # Django seems to silently swallow the ImportError under certain
+    # circumstances. Raise a generic exception explaining why we are
+    # unable to proceed.
+    raise Exception, 'FeinCMS requires PIL to be installed'
 
 from django import template
 from django.conf import settings
